#!/usr/bin/env python3
#
# Copyright (C) 2014 Geneviève Bastien <gbastien@versatic.net>
#
# SPDX-License-Identifier: GPL-2.0-only

import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *

NR_TESTS = 10
current_test = 1
print("1..{0}".format(NR_TESTS))

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("No sessiond running. Please make sure you are running this test with the \"run\" shell script and verify that the lttng tools are properly installed.")

session_info = create_session()
enable_ust_tracepoint_event(session_info, "ust_tests_td*")
start_session(session_info)

test_env = os.environ.copy()
test_env["LTTNG_UST_REGISTER_TIMEOUT"] = "-1"

td_process = subprocess.Popen(test_path + "type-declarations", stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, env=test_env)
td_process.wait()

print_test_result(td_process.returncode == 0, current_test, "Test application exited normally")
current_test += 1

stop_session(session_info)

# Check event fields using type declarations are present
try:
    babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
except FileNotFoundError:
    bail("Could not open babeltrace. Please make sure it is installed.")

event_lines = []
for event_line in babeltrace_process.stdout:
    event_line = event_line.decode('utf-8').replace("\n", "")
    event_lines.append(event_line)
babeltrace_process.wait()

print_test_result(babeltrace_process.returncode == 0, current_test, "Resulting trace is readable")
current_test += 1

if babeltrace_process.returncode != 0:
    bail("Unreadable trace; can't proceed with analysis.")

print_test_result(len(event_lines) == 5, current_test, "Correct number of events found in resulting trace")
current_test += 1

if len(event_lines) != 5:
    bail("Unexpected number of events found in resulting trace (" + session_info.trace_path + ")." )

match = re.search(r".*ust_tests_td:(.*):.*enumfield = \( \"(.*)\" :.*enumfield_bis = \( \"(.*)\" :.*enumfield_third = .*:.*", event_lines[0])
print_test_result(match is not None and match.group(1) == "tptest", current_test,\
                      "First tracepoint is present")
current_test += 1

print_test_result(match is not None and match.group(2) == "zero", current_test,\
                      "First tracepoint's enum value maps to zero")
current_test += 1

print_test_result(match is not None and match.group(3) == "one", current_test,\
                      "First tracepoint's second enum value maps to one")
current_test += 1

match = re.search(r".*ust_tests_td:(.*):.*enumfield = \( \"(.*)\" :.*", event_lines[1])
print_test_result(match is not None and match.group(1) == "tptest_bis", current_test,\
                      "Second tracepoint is present")
current_test += 1

print_test_result(match is not None and match.group(2) == "zero", current_test,\
                      "Second tracepoint's enum value maps to zero")
current_test += 1

match = re.search(r".*ust_tests_td:(.*):.*enumfield = \( \"(.*)\" :.*enumfield_bis = \( \"(.*)\" .*", event_lines[2])

print_test_result(match is not None and match.group(2) == "one", current_test,\
                      "Third tracepoint's enum value maps to one")
current_test += 1

print_test_result('{ zero = ( "zero" : container = 0 ), two = ( "two" : container = 2 ), three = ( "three" : container = 3 ), fifteen = ( "ten_to_twenty" : container = 15 ), twenty_one = ( "twenty_one" : container = 21 ) }' in event_lines[4],
                  current_test, 'Auto-incrementing enum values are correct')

shutil.rmtree(session_info.tmp_directory)



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only

import uuid
import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *


NR_TESTS = 6
current_test = 1
print("1..{0}".format(NR_TESTS))

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("No sessiond running. Please make sure you are running this test with the \"run\" shell script and verify that the lttng tools are properly installed.")

session_info = create_session()
enable_ust_tracepoint_event(session_info, "*")
start_session(session_info)


parent_pid = None
daemon_pid = None
daemon_process = subprocess.Popen(test_path + "daemon", stdout=subprocess.PIPE)
for line in daemon_process.stdout:
    name, pid = line.decode('utf-8').split()
    if name == "child_pid":
        daemon_pid = int(pid)
    if name == "parent_pid":
        parent_pid = int(pid)

daemon_process_return_code = daemon_process.wait()

if parent_pid is None or daemon_pid is None:
    bail("Unexpected output received from daemon test executable." + str(daemon_process_output))

print_test_result(daemon_process_return_code == 0, current_test, "Successful call to daemon() and normal exit")
current_test += 1

if daemon_process_return_code != 0:
    bail("Could not trigger tracepoints successfully. Abandoning test.")

stop_session(session_info)

try:
    babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE)
except FileNotFoundError:
    bail("Could not open babeltrace. Please make sure it is installed.")

before_daemon_event_found = False
before_daemon_event_pid = -1
after_daemon_event_found = False
after_daemon_event_pid = -1

for event_line in babeltrace_process.stdout:
    event_line = event_line.decode('utf-8').replace("\n", "")

    if re.search(r"before_daemon", event_line) is not None:
        if before_daemon_event_found:
            bail("Multiple instances of the before_daemon event found. Please make sure only one instance of this test is runnning.")
        before_daemon_event_found = True
        match = re.search(r"(?<=pid = )\d+", event_line)

        if match is not None:
            before_daemon_event_pid = int(match.group(0))

    if re.search(r"after_daemon", event_line) is not None:
        if after_daemon_event_found:
            bail("Multiple instances of the after_daemon event found. Please make sure only one instance of this test is runnning.")
        after_daemon_event_found = True
        match = re.search(r"(?<=pid = )\d+", event_line)

        if match is not None:
            after_daemon_event_pid = int(match.group(0))
babeltrace_process.wait()

print_test_result(babeltrace_process.returncode == 0, current_test, "Resulting trace is readable")
current_test += 1

if babeltrace_process.returncode != 0:
    bail("Unreadable trace; can't proceed with analysis.")

print_test_result(before_daemon_event_found, current_test, "before_daemon event found in resulting trace")
current_test += 1
print_test_result(before_daemon_event_pid == parent_pid, current_test, "Parent pid reported in trace is correct")
current_test += 1
print_test_result(before_daemon_event_found, current_test, "after_daemon event found in resulting trace")
current_test += 1
print_test_result(after_daemon_event_pid == daemon_pid, current_test, "Daemon pid reported in trace is correct")
current_test += 1

shutil.rmtree(session_info.tmp_directory)



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
# Copyright (C) 2015 Antoine Busque <abusque@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only

import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *


have_dlmopen = (os.environ.get('LTTNG_TOOLS_HAVE_DLMOPEN') == '1')


NR_TESTS = 14
current_test = 1
print("1..{0}".format(NR_TESTS))

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("""No sessiond running. Please make sure you are running this test
    with the "run" shell script and verify that the lttng tools are
    properly installed.""")

session_info = create_session()
enable_ust_tracepoint_event(session_info, "*")
start_session(session_info)

test_env = os.environ.copy()
test_env["LD_PRELOAD"] = test_env.get("LD_PRELOAD", "") + ":liblttng-ust-dl.so.0"
test_env["LD_LIBRARY_PATH"] = test_env.get("LD_LIBRARY_PATH", "") + ":" + test_path
test_process = subprocess.Popen(test_path + "prog",
                                stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL,
                                env=test_env)
test_process.wait()

print_test_result(test_process.returncode == 0, current_test, "Test application exited normally")
current_test += 1

stop_session(session_info)

# Check for dl events in the resulting trace
try:
    babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
except FileNotFoundError:
    bail("Could not open babeltrace. Please make sure it is installed.", session_info)

dlopen_event_found = 0
dlmopen_event_found = 0
build_id_event_found = 0
debug_link_event_found = 0
dlclose_event_found = 0
load_event_found = 0
load_build_id_event_found = 0
load_debug_link_event_found = 0
unload_event_found = 0
load_libfoo_found = 0
load_libbar_found = 0
load_libzzz_found = 0

for event_line in babeltrace_process.stdout:

    event_line = event_line.decode('utf-8').replace("\n", "")
    if re.search(r".*lttng_ust_dl:dlopen.*", event_line) is not None:
        dlopen_event_found += 1
    elif re.search(r".*lttng_ust_dl:dlmopen.*", event_line) is not None:
        dlmopen_event_found += 1
    elif re.search(r".*lttng_ust_dl:build_id.*", event_line) is not None:
        build_id_event_found += 1
    elif re.search(r".*lttng_ust_dl:debug_link.*", event_line) is not None:
        debug_link_event_found += 1
    elif re.search(r".*lttng_ust_dl:dlclose.*", event_line) is not None:
        dlclose_event_found += 1
    elif re.search(r".*lttng_ust_lib:build_id.*", event_line) is not None:
        load_build_id_event_found += 1
    elif re.search(r".*lttng_ust_lib:debug_link.*", event_line) is not None:
        load_debug_link_event_found += 1
    elif re.search(r".*lttng_ust_lib:unload.*", event_line) is not None:
        unload_event_found += 1
    elif re.search(r".*lttng_ust_lib:load.*", event_line) is not None:
        load_event_found += 1
        if re.search(r".*lttng_ust_lib:load.*libfoo.*", event_line) is not None:
            load_libfoo_found += 1
        elif re.search(r".*lttng_ust_lib:load.*libbar.*", event_line) is not None:
            load_libbar_found += 1
        elif re.search(r".*lttng_ust_lib:load.*libzzz.*", event_line) is not None:
            load_libzzz_found += 1

babeltrace_process.wait()

print_test_result(babeltrace_process.returncode == 0, current_test, "Resulting trace is readable")
current_test += 1

print_test_result(dlopen_event_found > 0, current_test, "lttng_ust_dl:dlopen event found in resulting trace")
current_test += 1

if have_dlmopen:
    print_test_result(dlmopen_event_found > 0, current_test, "lttng_ust_dl:dlmopen event found in resulting trace")
else:
    skip_test(current_test, 'dlmopen() is not available')

current_test += 1

print_test_result(build_id_event_found > 0, current_test, "lttng_ust_dl:build_id event found in resulting trace")
current_test += 1

print_test_result(debug_link_event_found > 0, current_test, "lttng_ust_dl:debug_link event found in resulting trace")
current_test += 1

print_test_result(dlclose_event_found > 0, current_test, "lttng_ust_dl:dlclose event found in resulting trace")
current_test += 1

print_test_result(load_event_found > 0, current_test, "lttng_ust_lib:load event found in resulting trace")
current_test += 1

print_test_result(load_build_id_event_found > 0, current_test, "lttng_ust_lib:build_id event found in resulting trace")
current_test += 1

print_test_result(load_debug_link_event_found > 0, current_test, "lttng_ust_lib:debug_link event found in resulting trace")
current_test += 1

print_test_result(unload_event_found == 3, current_test, "lttng_ust_lib:unload event found 3 times in resulting trace")
current_test += 1

print_test_result(load_libfoo_found == 1, current_test, "lttng_ust_lib:load libfoo.so event found once in resulting trace")
current_test += 1

print_test_result(load_libbar_found == 1, current_test, "lttng_ust_lib:load libbar.so event found once in resulting trace")
current_test += 1

print_test_result(load_libzzz_found == 1, current_test, "lttng_ust_lib:load libzzz.so event found once in resulting trace")
current_test += 1

shutil.rmtree(session_info.tmp_directory)



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only

import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *


NR_TESTS = 6
current_test = 1
print("1..{0}".format(NR_TESTS))

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("No sessiond running. Please make sure you are running this test with the \"run\" shell script and verify that the lttng tools are properly installed.")

session_info = create_session()
enable_ust_tracepoint_event(session_info, "ust_tests_fork*")
start_session(session_info)

fork_process = subprocess.Popen([test_path + "fork", test_path + "fork2"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
parent_pid = -1
child_pid = -1
for line in fork_process.stdout:
    line = line.decode('utf-8').replace("\n", "")
    match = re.search(r"child_pid (\d+)", line)
    if match:
        child_pid = match.group(1)
    match = re.search(r"parent_pid (\d+)", line)
    if match:
        parent_pid = match.group(1)

fork_process.wait()

print_test_result(fork_process.returncode == 0, current_test, "Fork test application exited normally")
current_test += 1

stop_session(session_info)

# Check both events (normal exit and suicide messages) are present in the resulting trace
try:
    babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
except FileNotFoundError:
    bail("Could not open babeltrace. Please make sure it is installed.", session_info)

event_lines = []
for event_line in babeltrace_process.stdout:
    event_line = event_line.decode('utf-8').replace("\n", "")
    if re.search(r"warning", event_line) is not None or re.search(r"error", event_line) is not None:
        print( "# " + event_line )
    else:
        event_lines.append(event_line)

babeltrace_process.wait()

print_test_result(babeltrace_process.returncode == 0, current_test, "Resulting trace is readable")
current_test += 1

if babeltrace_process.returncode != 0:
    bail("Unreadable trace; can't proceed with analysis.", session_info)

event_before_fork = False
event_after_fork_parent = False
event_after_fork_child = False
event_after_exec = False

for event_line in event_lines:
    match = re.search(r".*pid = (\d+)", event_line)
    if match is not None:
        event_pid = match.group(1)
    else:
        continue

    if re.search(r"before_fork", event_line):
        event_before_fork = (event_pid == parent_pid)
    if re.search(r"after_fork_parent", event_line):
        event_after_fork_parent = (event_pid == parent_pid)
    if re.search(r"after_fork_child", event_line):
        event_after_fork_child = (event_pid == child_pid)
    if re.search(r"after_exec", event_line):
        event_after_exec = (event_pid == child_pid)

print_test_result(event_before_fork, current_test, "before_fork event logged by parent process found in trace")
current_test += 1
print_test_result(event_after_fork_parent, current_test, "after_fork_parent event logged by parent process found in trace")
current_test += 1
print_test_result(event_after_fork_child, current_test, "after_fork_child event logged by child process found in trace")
current_test += 1
print_test_result(event_after_exec, current_test, "after_exec event logged by child process found in trace")
current_test += 1

shutil.rmtree(session_info.tmp_directory)



#!/usr/bin/env python3
#
# Copyright (C) 2017 Francis Deslauriers <francis.deslauriers@efficios.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import sys
import bisect
import subprocess
import re

def addr2line(executable, addr):
    """
        Uses binutils' addr2line to get function containing a given address
    """
    cmd =['addr2line']

    cmd += ['-e', executable]

    # Print function names
    cmd += ['--functions']

    # Expand inlined functions
    cmd += ['--addresses', addr]

    status = subprocess.run(cmd, stdout=subprocess.PIPE, check=True)

    addr2line_output = status.stdout.decode("utf-8").splitlines()
    # addr2line's output is made of 3-tuples:
    #   - address
    #   - function name
    #   - source location
    if len(addr2line_output) % 3 != 0:
        raise Exception('Unexpected addr2line output:\n\t{}'.format('\n\t'.join(addr2line_output)))

    function_names = []
    for address_line_number in range(0, len(addr2line_output), 3):
        function_name = addr2line_output[address_line_number + 1]

        # Filter-out unresolved functions
        if "??" not in function_name:
            function_names.append(addr2line_output[address_line_number + 1])

    return function_names

def extract_user_func_names(executable, raw_callstack):
    """
        Given a callstack from the Babeltrace CLI output, returns a set
        containing the name of the functions. This assumes that the binary have
        not changed since the execution.
    """
    recorded_callstack = set()

    # Remove commas and split on spaces
    for index, addr in enumerate(raw_callstack.replace(',', '').split(' ')):
        # Consider only the elements starting with '0x' which are the
        # addresses recorded in the callstack
        if '0x' in addr[:2]:
            funcs = addr2line(executable, addr)
            recorded_callstack.update(funcs)

    return recorded_callstack

def extract_kernel_func_names(raw_callstack):
    """
        Given a callstack from the Babeltrace CLI output, returns a set
        containing the name of the functions.
        Uses the /proc/kallsyms procfile to find the symbol associated with an
        address. This function should only be used if the user is root or has
        access to /proc/kallsyms.
    """
    recorded_callstack = set()
    syms=[]
    addresses=[]
    # We read kallsyms file and save the output
    with open('/proc/kallsyms') as kallsyms_f:
        for line in kallsyms_f:
            line_tokens = line.split()
            addr = line_tokens[0]
            symbol = line_tokens[2]
            addresses.append(int(addr, 16))
            syms.append({'addr':int(addr, 16), 'symbol':symbol})

    # Save the address and symbol in a sorted list of tupple
    syms = sorted(syms, key=lambda k:k['addr'])
    # We save the list of addresses in a seperate sorted list to easily bisect
    # the closer address of a symbol.
    addresses = sorted(addresses)

    # Remove commas and split on spaces
    for addr in raw_callstack.replace(',', '').split(' '):
        if '0x' in addr[:2]:
            # Search the location of the address in the addresses list and
            # deference this location in the syms list to add the associated
            # symbol.
            loc = bisect.bisect(addresses, int(addr, 16))
            recorded_callstack.add(syms[loc-1]['symbol'])

    return recorded_callstack

# Regex capturing the callstack_user and callstack_kernel context
user_cs_rexp='.*callstack_user\ \=\ \[(.*)\]\ .*\}, \{.*\}'
kernel_cs_rexp='.*callstack_kernel\ \=\ \[(.*)\]\ .*\}, \{.*\}'

def main():
    """
        Reads a line from stdin and expect it to be a wellformed Babeltrace CLI
        output containing containing a callstack context of the domain passed
        as argument.
    """
    expected_callstack = set()
    recorded_callstack = set()
    cs_type=None

    if len(sys.argv) <= 2:
        print(sys.argv)
        raise ValueError('USAGE: ./{} (--kernel|--user EXE) FUNC-NAMES'.format(sys.argv[0]))

    # If the `--user` option is passed, save the next argument as the path
    # to the executable
    argc=1
    executable=None
    if sys.argv[argc] in '--kernel':
        rexp = kernel_cs_rexp
        cs_type='kernel'
    elif sys.argv[argc] in '--user':
        rexp = user_cs_rexp
        cs_type='user'
        argc+=1
        executable = sys.argv[argc]
    else:
        raise Exception('Unknown domain')

    argc+=1

    # Extract the function names that are expected to be found call stack of
    # the current events
    for func in sys.argv[argc:]:
        expected_callstack.add(func)

    # Read the tested line for STDIN
    event_line = None
    for line in sys.stdin:
        event_line = line
        break

    # Extract the userspace callstack context of the event
    m = re.match(rexp, event_line)

    # If there is no match, exit with error
    if m is None:
        raise re.error('Callstack not found in event line')
    else:
        raw_callstack = str(m.group(1))
        if cs_type in 'user':
            recorded_callstack=extract_user_func_names(executable, raw_callstack)
        elif cs_type in 'kernel':
            recorded_callstack=extract_kernel_func_names(raw_callstack)
        else:
            raise Exception('Unknown domain')

    # Verify that all expected function are present in the callstack
    for e in expected_callstack:
        if e not in recorded_callstack:
            raise Exception('Expected function name not found in recorded callstack')

    sys.exit(0)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only
#

import uuid
import os
import subprocess
import shutil
import sys
import tempfile

# Import lttng bindings generated in the current tree
lttng_bindings_path = os.path.dirname(os.path.abspath(__file__)) + "/"
for i in range(3):
    lttng_bindings_path = os.path.dirname(lttng_bindings_path)
lttng_bindings_path = lttng_bindings_path + "/extras/bindings/swig/python"
lttng_bindings_libs_path = lttng_bindings_path + "/.libs"
sys.path.append(lttng_bindings_path)
sys.path.append(lttng_bindings_libs_path)
from lttng import *


class SessionInfo:
    def __init__(self, handle, session_name, tmp_directory, channel_name):
        self.handle = handle
        self.name = session_name
        self.tmp_directory = tmp_directory
        self.trace_path = tmp_directory + "/" + session_name
        self.channel_name = channel_name

def bail(diag, session_info = None):
    print("Bail out!")
    print("#", diag)

    if session_info is not None:
        stop_session(session_info, True)

        if os.path.exists(session_info.tmp_directory):
            shutil.rmtree(session_info.tmp_directory)
    exit(-1)

def print_test_result(result, number, description):
    result_string = None
    if result is True:
        result_string = "ok"
    else:
        result_string = "not ok"

    result_string += " {0} - {1}".format(number, description)
    print(result_string)

def skip_test(number, description):
    print('ok {} # skip {}'.format(number, description))

def enable_ust_tracepoint_event(session_info, event_name):
    event = Event()
    event.name = event_name
    event.type = EVENT_TRACEPOINT
    event.loglevel = EVENT_LOGLEVEL_ALL
    res = enable_event(session_info.handle, event, session_info.channel_name)
    if res < 0:
        bail("Failed to enable userspace event " + event_name, session_info)

def create_session():
    dom = Domain()
    dom.type = DOMAIN_UST

    session_name = str(uuid.uuid1())
    tmp_directory = tempfile.mkdtemp()
    trace_path = tmp_directory + "/" + session_name

    res = create(session_name, trace_path)
    if res < 0:
        bail("Failed to create recording session.")

    channel = Channel()
    channel.name = "channel0"
    channel_set_default_attr(dom, channel.attr)

    han = Handle(session_name, dom)
    res = enable_channel(han, channel)

    session_info = SessionInfo(han, session_name, tmp_directory, channel.name)
    if res < 0:
        bail("Failed to enable channel " + channel.name, session_info)
    return session_info

def start_session(session_info):
    start(session_info.name)

def stop_session(session_info, bailing = False):
    # Workaround lttng-ctl outputing directly to stdout by spawning a subprocess.
    lttng_binary_path = os.path.dirname(os.path.abspath(__file__)) + "/"
    for i in range(3):
        lttng_binary_path = os.path.dirname(lttng_binary_path)
    lttng_binary_path = lttng_binary_path + "/src/bin/lttng/lttng"

    retcode = subprocess.call([lttng_binary_path, "stop", session_info.name], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if retcode != 0 and not bailing:
        bail("Unable to stop session " + session_info.name, session_info)
    destroy(session_info.name)



# Copyright (C) 2015 Philippe Proulx <pproulx@efficios.com>
# Copyright (C) 2014 David Goulet <dgoulet@efficios.com>
#
# SPDX-License-Identifier: LGPL-2.1-only

from __future__ import unicode_literals, print_function
import logging
import time
import sys
import argparse
import os


def _perror(msg):
    print(msg, file=sys.stderr)
    sys.exit(1)


try:
    import lttngust
except (ImportError) as e:
    _perror('lttngust package not found: {}'.format(e))


def _main():
    ev1 = logging.getLogger('python-ev-test1');
    ev2 = logging.getLogger('python-ev-test2');

    logging.basicConfig()

    parser = argparse.ArgumentParser()
    parser.add_argument('-n', '--nr-iter', required=True)
    parser.add_argument('-s', '--wait', required=True)
    parser.add_argument('-d', '--fire-debug-event', action="store_true")
    parser.add_argument('-e', '--fire-second-event', action="store_true")
    parser.add_argument('-r', '--ready-file')
    parser.add_argument('-g', '--go-file')
    args = parser.parse_args()

    nr_iter = int(args.nr_iter)
    wait_time = float(args.wait)
    fire_debug_ev = args.fire_debug_event
    fire_second_ev = args.fire_second_event

    ready_file = args.ready_file
    go_file = args.go_file

    if ready_file is not None and os.path.exists(ready_file):
        raise ValueError('Ready file already exist')

    if go_file is not None and os.path.exists(go_file):
        raise ValueError('Go file already exist. Review synchronization')

    if (ready_file is None) != (go_file is None):
        raise ValueError('--go-file and --ready-file need each others, review'
                'synchronization')


    # Inform that we are ready, if necessary
    if ready_file is not None:
        open(ready_file, 'a').close()

    # Wait for go, if necessary
    while go_file is not None and not os.path.exists(go_file):
        time.sleep(0.5)

    for i in range(nr_iter):
        ev1.info('{} fired [INFO]'.format(ev1.name))

        if fire_debug_ev:
            ev1.debug('{} fired [DEBUG]'.format(ev1.name))

        time.sleep(wait_time)

    if fire_second_ev:
        ev2.info('{} fired [INFO]'.format(ev2.name))

    if ready_file is not None:
        try:
            os.unlink(ready_file)
        except:
            print("Unexpected error on ready file unlink:", sys.exc_info()[0])
            raise


if __name__ == '__main__':
    _main()



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only

import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *


NR_TESTS = 4
current_test = 1
print("1..{0}".format(NR_TESTS))

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("No sessiond running. Please make sure you are running this test with the \"run\" shell script and verify that the lttng tools are properly installed.")

session_info = create_session()
enable_ust_tracepoint_event(session_info, "lttng_ust_libc*")
start_session(session_info)

malloc_process = subprocess.Popen(test_path + "prog", stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
malloc_process.wait()

print_test_result(malloc_process.returncode == 0, current_test, "Test application exited normally")
current_test += 1

stop_session(session_info)

# Check for malloc events in the resulting trace
try:
    babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
except FileNotFoundError:
    bail("Could not open babeltrace. Please make sure it is installed.", session_info)

malloc_event_found = False
free_event_found = False

for event_line in babeltrace_process.stdout:
    # Let babeltrace finish to get the return code
    if malloc_event_found and free_event_found:
        continue

    event_line = event_line.decode('utf-8').replace("\n", "")
    if re.search(r".*lttng_ust_libc:malloc.*", event_line) is not None:
        malloc_event_found = True

    if re.search(r".*lttng_ust_libc:free.*", event_line) is not None:
        free_event_found = True

babeltrace_process.wait()

print_test_result(babeltrace_process.returncode == 0, current_test, "Resulting trace is readable")
current_test += 1

print_test_result(malloc_event_found, current_test, "lttng_ust_libc:malloc event found in resulting trace")
current_test += 1

print_test_result(free_event_found, current_test, "lttng_ust_libc:free event found in resulting trace")
current_test += 1

shutil.rmtree(session_info.tmp_directory)



#!/usr/bin/env python3
#
# Copyright (C) 2016 Julien Desfossez <jdesfossez@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only
#

import argparse
import pprint
import sys
import time
import json

from collections import defaultdict

NSEC_PER_SEC = 1000000000

try:
    from babeltrace import TraceCollection
except ImportError:
    # quick fix for debian-based distros
    sys.path.append("/usr/local/lib/python%d.%d/site-packages" %
                    (sys.version_info.major, sys.version_info.minor))
    from babeltrace import TraceCollection


class TraceParser:
    def __init__(self, trace, pid):
        self.trace = trace
        self.pid = pid

        # This dictionnary holds the results of each testcases of a test.
        # Its layout is the following:
        #       self.expect={
        #                       'event_name_1': {'check_1': 0, 'check_2: 1},
        #                       'event_name_2': {'check_1': 1}
        #                   }
        # Each test classes checks the payload of different events. Each of
        # those checks are stored in a event_name specific dictionnary in this
        # data structure.
        self.expect = defaultdict(lambda : defaultdict(int))

        # This dictionnary holds the value recorded in the trace that are
        # tested. Its content is use to print the values that caused a test to
        # fail.
        self.recorded_values = {}

    def ns_to_hour_nsec(self, ns):
        d = time.localtime(ns/NSEC_PER_SEC)
        return "%02d:%02d:%02d.%09d" % (d.tm_hour, d.tm_min, d.tm_sec,
                                        ns % NSEC_PER_SEC)

    def parse(self):
        # iterate over all the events
        for event in self.trace.events:
            if self.pid is not None and event["pid"] != self.pid:
                continue

            method_name = "handle_%s" % event.name.replace(":", "_").replace(
                "+", "_")
            # call the function to handle each event individually
            if hasattr(TraceParser, method_name):
                func = getattr(TraceParser, method_name)
                func(self, event)

        ret = 0
        # For each event of the test case, check all entries for failed
        for event_name, event_results in self.expect.items():
            for val in event_results.keys():
                if self.expect[event_name][val] == 0:
                    print("%s not validated" % val)
                    print("Values of the local variables of this test:")
                    # using pprint for pretty printing the dictionnary
                    pprint.pprint(self.recorded_values[event_name])
                    ret = 1

        return ret

    # epoll_ctl
    def handle_compat_syscall_entry_epoll_ctl(self, event):
        self.epoll_ctl_entry(event)

    def handle_compat_syscall_exit_epoll_ctl(self, event):
        self.epoll_ctl_exit(event)

    def handle_syscall_entry_epoll_ctl(self, event):
        self.epoll_ctl_entry(event)

    def handle_syscall_exit_epoll_ctl(self, event):
        self.epoll_ctl_exit(event)

    def epoll_ctl_entry(self, event):
        pass

    def epoll_ctl_exit(self, event):
        pass

    # epoll_wait + epoll_pwait
    def handle_compat_syscall_entry_epoll_wait(self, event):
        self.epoll_wait_entry(event)

    def handle_compat_syscall_exit_epoll_wait(self, event):
        self.epoll_wait_exit(event)

    def handle_syscall_entry_epoll_wait(self, event):
        self.epoll_wait_entry(event)

    def handle_syscall_exit_epoll_wait(self, event):
        self.epoll_wait_exit(event)

    def handle_compat_syscall_entry_epoll_pwait(self, event):
        self.epoll_pwait_entry(event)

    def handle_compat_syscall_exit_epoll_pwait(self, event):
        self.epoll_pwait_exit(event)

    def handle_syscall_entry_epoll_pwait(self, event):
        self.epoll_pwait_entry(event)

    def handle_syscall_exit_epoll_pwait(self, event):
        self.epoll_pwait_exit(event)

    def epoll_wait_entry(self, event):
        pass

    def epoll_wait_exit(self, event):
        pass

    def epoll_pwait_entry(self, event):
        self.epoll_wait_entry(event)

    def epoll_pwait_exit(self, event):
        self.epoll_wait_exit(event)

    ## poll + ppoll
    def handle_compat_syscall_entry_poll(self, event):
        self.poll_entry(event)

    def handle_compat_syscall_exit_poll(self, event):
        self.poll_exit(event)

    def handle_syscall_entry_poll(self, event):
        self.poll_entry(event)

    def handle_syscall_exit_poll(self, event):
        self.poll_exit(event)

    def handle_compat_syscall_entry_ppoll(self, event):
        self.poll_entry(event)

    def handle_compat_syscall_exit_ppoll(self, event):
        self.poll_exit(event)

    def handle_syscall_entry_ppoll(self, event):
        self.poll_entry(event)

    def handle_syscall_exit_ppoll(self, event):
        self.poll_exit(event)

    def poll_entry(self, event):
        pass

    def poll_exit(self, event):
        pass

    # epoll_create
    def handle_compat_syscall_entry_epoll_create1(self, event):
        self.epoll_create_entry(event)

    def handle_compat_syscall_exit_epoll_create1(self, event):
        self.epoll_create_exit(event)

    def handle_compat_syscall_entry_epoll_create(self, event):
        self.epoll_create_entry(event)

    def handle_compat_syscall_exit_epoll_create(self, event):
        self.epoll_create_exit(event)

    def handle_syscall_entry_epoll_create1(self, event):
        self.epoll_create_entry(event)

    def handle_syscall_exit_epoll_create1(self, event):
        self.epoll_create_exit(event)

    def handle_syscall_entry_epoll_create(self, event):
        self.epoll_create_entry(event)

    def handle_syscall_exit_epoll_create(self, event):
        self.epoll_create_exit(event)

    def epoll_create_entry(self, event):
        pass

    def epoll_create_exit(self, event):
        pass

    # select + pselect6
    def handle_syscall_entry_pselect6(self, event):
        self.select_entry(event)

    def handle_syscall_exit_pselect6(self, event):
        self.select_exit(event)

    def handle_compat_syscall_entry_pselect6(self, event):
        self.select_entry(event)

    def handle_compat_syscall_exit_pselect6(self, event):
        self.select_exit(event)

    def handle_syscall_entry_select(self, event):
        self.select_entry(event)

    def handle_syscall_exit_select(self, event):
        self.select_exit(event)

    def handle_compat_syscall_entry_select(self, event):
        self.select_entry(event)

    def handle_compat_syscall_exit_select(self, event):
        self.select_exit(event)

    def select_entry(self, event):
        pass

    def select_exit(self, event):
        pass


class Test1(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])

        # Values expected in the trace
        self.epoll_wait_fd = validation_args['epoll_wait_fd']
        self.epoll_pwait_fd = validation_args['epoll_pwait_fd']

        self.expect["select_entry"]["select_in_fd0"] = 0
        self.expect["select_entry"]["select_in_fd1023"] = 0
        self.expect["select_exit"]["select_out_fd0"] = 0
        self.expect["select_exit"]["select_out_fd1023"] = 0
        self.expect["poll_entry"]["poll_in_nfds1"] = 0
        self.expect["poll_exit"]["poll_out_nfds1"] = 0
        self.expect["epoll_ctl_entry"]["epoll_ctl_in_add"] = 0
        self.expect["epoll_ctl_exit"]["epoll_ctl_out_ok"] = 0
        self.expect["epoll_wait_entry"]["epoll_wait_in_ok"] = 0
        self.expect["epoll_wait_exit"]["epoll_wait_out_fd0"] = 0
        self.expect["epoll_pwait_entry"]["epoll_pwait_in_ok"] = 0
        self.expect["epoll_pwait_exit"]["epoll_pwait_out_fd0"] = 0

    def select_entry(self, event):
        n = event["n"]
        overflow = event["overflow"]
        readfd_0 = event["readfds"][0]

        # check that the FD 0 is actually set in the readfds
        if n == 1 and readfd_0 == 1:
            self.expect["select_entry"]["select_in_fd0"] = 1
        if n == 1023:
            readfd_127 = event["readfds"][127]
            writefd_127 = event["writefds"][127]
            exceptfd_127 = event["exceptfds"][127]

            # check that the FD 1023 is actually set in the readfds
            if readfd_127 == 0x40 and writefd_127 == 0 and \
                    exceptfd_127 == 0 and overflow == 0:
                self.expect["select_entry"]["select_in_fd1023"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_entry"] = locals()

    def select_exit(self, event):
        ret = event["ret"]
        tvp = event["tvp"]
        overflow = event["overflow"]
        _readfds_length = event["_readfds_length"]

        if ret == 1:
            # check that the FD 0 is actually set in the readfds
            readfd_0 = event["readfds"][0]

            if readfd_0 == 1:
                self.expect["select_exit"]["select_out_fd0"] = 1
            # check that the FD 1023 is actually set in the readfds
            if _readfds_length == 128:
                readfd_127 = event["readfds"][127]
                writefd_127 = event["writefds"][127]
                exceptfd_127 = event["exceptfds"][127]
                if readfd_127 == 0x40 and writefd_127 == 0 and \
                        exceptfd_127 == 0 and tvp == 0:
                    self.expect["select_exit"]["select_out_fd1023"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_exit"] = locals()

    def poll_entry(self, event):
        nfds = event["nfds"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # check that only one FD is set, that it has the POLLIN flag and that
        # the raw value matches the events bit field.
        if nfds == 1 and fds_length == 1:
            fd_0 = event["fds"][0]
            if fd_0["raw_events"] == 0x3 and fd_0["events"]["POLLIN"] == 1 and \
                    fd_0["events"]["padding"] == 0:
                self.expect["poll_entry"]["poll_in_nfds1"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_entry"] = locals()

    def poll_exit(self, event):
        ret = event["ret"]
        fds_length = event["fds_length"]

        # check that only one FD is set, that it has the POLLIN flag and that
        # the raw value matches the events bit field.
        if ret == 1 and fds_length == 1:
            fd_0 = event["fds"][0]
            if fd_0["raw_events"] == 0x1 and fd_0["events"]["POLLIN"] == 1 and \
                fd_0["events"]["padding"] == 0:
                self.expect["poll_exit"]["poll_out_nfds1"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_exit"] = locals()

    def epoll_ctl_entry(self, event):
        epfd = event["epfd"]
        op_enum = event["op_enum"]
        fd = event["fd"]
        _event = event["event"]

        # check that we have FD 0 waiting for EPOLLIN|EPOLLPRI and that
        # data.fd = 0
        if (epfd == self.epoll_wait_fd or epfd == self.epoll_pwait_fd) and 'EPOLL_CTL_ADD' == op_enum and fd == 0 and \
                _event["data_union"]["fd"] == 0 and \
                _event["events"]["EPOLLIN"] == 1 and \
                _event["events"]["EPOLLPRI"] == 1:
            self.expect["epoll_ctl_entry"]["epoll_ctl_in_add"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_ctl_entry"] = locals()

    def epoll_ctl_exit(self, event):
        ret = event["ret"]

        if ret == 0:
            self.expect["epoll_ctl_exit"]["epoll_ctl_out_ok"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_ctl_exit"] = locals()

    def epoll_wait_entry(self, event):
        epfd = event["epfd"]
        maxevents = event["maxevents"]
        timeout = event["timeout"]

        if epfd == self.epoll_wait_fd and maxevents == 1 and timeout == -1:
            self.expect["epoll_wait_entry"]["epoll_wait_in_ok"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_entry"] = locals()

    def epoll_wait_exit(self, event):
        ret = event["ret"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # check that FD 0 returned with EPOLLIN and the right data.fd
        if ret == 1 and fds_length == 1:
            fd_0 = event["fds"][0]
            if overflow == 0 and  fd_0["data_union"]["fd"] == 0 and \
                fd_0["events"]["EPOLLIN"] == 1:
                self.expect["epoll_wait_exit"]["epoll_wait_out_fd0"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_exit"] = locals()

    def epoll_pwait_entry(self, event):
        epfd = event["epfd"]
        maxevents = event["maxevents"]
        timeout = event["timeout"]

        if epfd == self.epoll_pwait_fd and maxevents == 1 and timeout == -1:
            self.expect["epoll_pwait_entry"]["epoll_pwait_in_ok"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_pwait_entry"] = locals()

    def epoll_pwait_exit(self, event):
        ret = event["ret"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # check that FD 0 returned with EPOLLIN and the right data.fd
        if ret == 1 and fds_length == 1:
            fd_0 = event["fds"][0]
            if overflow == 0 and  fd_0["data_union"]["fd"] == 0 and \
                fd_0["events"]["EPOLLIN"] == 1:
                self.expect["epoll_pwait_exit"]["epoll_pwait_out_fd0"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_pwait_exit"] = locals()

class Test2(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])
        self.expect["select_entry"]["select_timeout_in_fd0"] = 0
        self.expect["select_entry"]["select_timeout_in_fd1023"] = 0
        self.expect["select_exit"]["select_timeout_out"] = 0
        self.expect["poll_entry"]["poll_timeout_in"] = 0
        self.expect["poll_exit"]["poll_timeout_out"] = 0
        self.expect["epoll_ctl_entry"]["epoll_ctl_timeout_in_add"] = 0
        self.expect["epoll_ctl_exit"]["epoll_ctl_timeout_out_ok"] = 0
        self.expect["epoll_wait_entry"]["epoll_wait_timeout_in"] = 0
        self.expect["epoll_wait_exit"]["epoll_wait_timeout_out"] = 0

    def select_entry(self, event):
        n = event["n"]
        tvp = event["tvp"]

        if n == 1 and tvp != 0:
            self.expect["select_entry"]["select_timeout_in_fd0"] = 1
        if n == 1023:
            readfd_127 = event["readfds"][127]
            writefd_127 = event["writefds"][127]
            exceptfd_127 = event["exceptfds"][127]

            if readfd_127 == 0x40 and writefd_127 == 0 and \
                    exceptfd_127 == 0 and tvp != 0:
                self.expect["select_entry"]["select_timeout_in_fd1023"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_entry"] = locals()

    def select_exit(self, event):
        ret = event["ret"]
        tvp = event["tvp"]

        if ret == 0 and tvp != 0:
            self.expect["select_exit"]["select_timeout_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_exit"] = locals()

    def poll_entry(self, event):
        nfds = event["nfds"]
        fds_length = event["fds_length"]

        # check that we wait on FD 0 for POLLIN and that the raw_events
        # field matches the value of POLLIN
        if nfds == 1 and fds_length == 1:
            fd_0 = event["fds"][0]
            if fd_0["raw_events"] == 0x3 and \
                    fd_0["events"]["POLLIN"] == 1 and \
                    fd_0["events"]["padding"] == 0:
                self.expect["poll_entry"]["poll_timeout_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_entry"] = locals()

    def poll_exit(self, event):
        ret = event["ret"]
        nfds = event["nfds"]
        fds_length = event["fds_length"]

        if ret == 0 and nfds == 1 and fds_length == 0:
            self.expect["poll_exit"]["poll_timeout_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_exit"] = locals()

    def epoll_ctl_entry(self, event):
        op_enum = event["op_enum"]
        _event = event["event"]

        # make sure we see a EPOLLIN|EPOLLPRI
        if op_enum == "EPOLL_CTL_ADD" and \
                _event["events"]["EPOLLIN"] == 1 and \
                _event["events"]["EPOLLPRI"] == 1:
            self.expect["epoll_ctl_entry"]["epoll_ctl_timeout_in_add"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_ctl_entry"] = locals()

    def epoll_ctl_exit(self, event):
        ret = event["ret"]

        if ret == 0:
            self.expect["epoll_ctl_exit"]["epoll_ctl_timeout_out_ok"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_ctl_exit"] = locals()

    def epoll_wait_entry(self, event):
        maxevents = event["maxevents"]
        timeout = event["timeout"]

        if maxevents == 1 and timeout == 1:
            self.expect["epoll_wait_entry"]["epoll_wait_timeout_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_entry"] = locals()

    def epoll_wait_exit(self, event):
        ret = event["ret"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        if ret == 0 and fds_length == 0 and overflow == 0:
            self.expect["epoll_wait_exit"]["epoll_wait_timeout_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_exit"] = locals()


class Test3(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])
        self.expect["select_entry"]["select_invalid_fd_in"] = 0
        self.expect["select_exit"]["select_invalid_fd_out"] = 0

    def select_entry(self, event):
        n = event["n"]
        overflow = event["overflow"]

        if n > 0 and overflow == 0:
            self.expect["select_entry"]["select_invalid_fd_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_entry"] = locals()

    def select_exit(self, event):
        ret = event["ret"]
        overflow = event["overflow"]
        _readfds_length = event["_readfds_length"]

        # make sure the event has a ret field equal to -EBADF
        if ret == -9 and overflow == 0 and _readfds_length == 0:
            self.expect["select_exit"]["select_invalid_fd_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_exit"] = locals()


class Test4(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])
        self.expect["poll_entry"]["big_poll_in"] = 0
        self.expect["poll_exit"]["big_poll_out"] = 0

    def poll_entry(self, event):
        nfds = event["nfds"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # test of big list of FDs and the behaviour of the overflow
        if nfds == 2047 and fds_length == 512 and overflow == 1:
            fd_0 = event["fds"][0]
            fd_511 = event["fds"][511]
            if fd_0["raw_events"] == 0x3 and fd_0["events"]["POLLIN"] == 1 and \
                    fd_0["events"]["padding"] == 0 and \
                    fd_511["events"]["POLLIN"] == 1 and \
                    fd_511["events"]["POLLPRI"] == 1:
                self.expect["poll_entry"]["big_poll_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_entry"] = locals()

    def poll_exit(self, event):
        ret = event["ret"]
        nfds = event["nfds"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # test of big list of FDs and the behaviour of the overflow
        if ret == 2047 and nfds == 2047 and fds_length == 512 and overflow == 1:
                fd_0 = event["fds"][0]
                fd_511 = event["fds"][511]
                if fd_0["events"]["POLLIN"] == 1 and fd_511["events"]["POLLIN"] == 1:
                    self.expect["poll_exit"]["big_poll_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_exit"] = locals()

class Test5(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])
        self.expect["poll_entry"]["poll_overflow_in"] = 0
        self.expect["poll_exit"]["poll_overflow_out"] = 0

    def poll_entry(self, event):
        nfds = event["nfds"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # test that event in valid even though the target buffer is too small
        # and the program segfaults
        if nfds == 100 and fds_length == 100 and overflow == 0:
            fd_0 = event["fds"][0]
            if fd_0["events"]["POLLIN"] == 1:
                self.expect["poll_entry"]["poll_overflow_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_entry"] = locals()

    def poll_exit(self, event):
        nfds = event["nfds"]
        overflow = event["overflow"]

        # test that event in valid even though the target buffer is too small
        # and the program segfaults
        if nfds == 100 and overflow == 0:
            self.expect["poll_exit"]["poll_overflow_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_exit"] = locals()


class Test6(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])
        self.expect["select_entry"]["pselect_invalid_in"] = 0
        self.expect["select_exit"]["pselect_invalid_out"] = 0

    def select_entry(self, event):
        n = event["n"]
        overflow = event["overflow"]
        _readfds_length = event["_readfds_length"]

        # test that event in valid even though the target buffer pointer is
        # invalid and the program segfaults
        if n == 1 and overflow == 0 and _readfds_length == 0:
            self.expect["select_entry"]["pselect_invalid_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_entry"] = locals()

    def select_exit(self, event):
        ret = event["ret"]
        overflow = event["overflow"]
        _readfds_length = event["_readfds_length"]

        # test that event in valid even though the target buffer pointer is
        # invalid and the program segfaults
        if ret == -14 and overflow == 0 and _readfds_length == 0:
            self.expect["select_exit"]["pselect_invalid_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["select_exit"] = locals()


class Test7(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])
        self.expect["poll_entry"]["poll_max_in"] = 0
        self.expect["poll_exit"]["poll_max_out"] = 0

    def poll_entry(self, event):
        nfds = event["nfds"]
        overflow = event["overflow"]

        # check the proper working of INT_MAX maxevent value
        if nfds == 4294967295 and overflow == 1:
            self.expect["poll_entry"]["poll_max_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_entry"] = locals()


    def poll_exit(self, event):
        ret = event["ret"]
        nfds = event["nfds"]
        overflow = event["overflow"]

        # check the proper working of UINT_MAX maxevent value
        if ret == -22 and nfds == 4294967295 and overflow == 0:
            self.expect["poll_exit"]["poll_max_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["poll_exit"] = locals()


class Test8(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])

        # Values expected in the trace
        self.epoll_fd = validation_args['epollfd']

        self.expect["epoll_wait_entry"]["epoll_wait_invalid_in"] = 0
        self.expect["epoll_wait_exit"]["epoll_wait_invalid_out"] = 0

    def epoll_wait_entry(self, event):
        epfd = event["epfd"]
        maxevents = event["maxevents"]
        timeout = event["timeout"]

        # test that event in valid even though the target buffer pointer is
        # invalid and the program segfaults
        if epfd == self.epoll_fd and maxevents == 1 and timeout == -1:
            self.expect["epoll_wait_entry"]["epoll_wait_invalid_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_entry"] = locals()

    def epoll_wait_exit(self, event):
        ret = event["ret"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # test that event in valid even though the target buffer pointer is
        # invalid and the program segfaults
        if ret == -14 and fds_length == 0 and overflow == 0:
            self.expect["epoll_wait_exit"]["epoll_wait_invalid_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_exit"] = locals()


class Test9(TraceParser):
    def __init__(self, trace, validation_args):
        super().__init__(trace, validation_args['pid'])

        # Values expected in the trace
        self.epoll_fd = validation_args['epollfd']

        self.expect["epoll_wait_entry"]["epoll_wait_max_in"] = 0
        self.expect["epoll_wait_exit"]["epoll_wait_max_out"] = 0

    def epoll_wait_entry(self, event):
        epfd = event["epfd"]
        maxevents = event["maxevents"]
        timeout = event["timeout"]

        # check the proper working of INT_MAX maxevent value
        if epfd == self.epoll_fd and maxevents == 2147483647 and timeout == -1:
            self.expect["epoll_wait_entry"]["epoll_wait_max_in"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_entry"] = locals()

    def epoll_wait_exit(self, event):
        ret = event["ret"]
        fds_length = event["fds_length"]
        overflow = event["overflow"]

        # check the proper working of INT_MAX maxevent value
        if ret == -22 and fds_length == 0 and overflow == 0:
            self.expect["epoll_wait_exit"]["epoll_wait_max_out"] = 1

        # Save values of local variables to print in case of test failure
        self.recorded_values["epoll_wait_exit"] = locals()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Trace parser')
    parser.add_argument('path', metavar="<path/to/trace>", help='Trace path')
    parser.add_argument('-t', '--test', type=int, help='Test to validate')
    parser.add_argument('-o', '--validation-file', type=str, help='Validation file path')
    args = parser.parse_args()

    if not args.test:
        print("Need to pass a test to validate (--test/-t)")
        sys.exit(1)

    if not args.validation_file:
        print("Need to pass the test validation file (--validation-file/-o)")
        sys.exit(1)

    traces = TraceCollection()
    handle = traces.add_traces_recursive(args.path, "ctf")
    if handle is None:
        sys.exit(1)

    with open(args.validation_file) as f:
        try:
            test_validation_args = json.load(f)
        except Exception as e:
            print('Failed to parse validation file: ' + str(e))
            sys.exit(1)

    t = None

    if args.test == 1:
        t = Test1(traces, test_validation_args)
    elif args.test == 2:
        t = Test2(traces, test_validation_args)
    elif args.test == 3:
        t = Test3(traces, test_validation_args)
    elif args.test == 4:
        t = Test4(traces, test_validation_args)
    elif args.test == 5:
        t = Test5(traces, test_validation_args)
    elif args.test == 6:
        t = Test6(traces, test_validation_args)
    elif args.test == 7:
        t = Test7(traces, test_validation_args)
    elif args.test == 8:
        t = Test8(traces, test_validation_args)
    elif args.test == 9:
        t = Test9(traces, test_validation_args)
    elif args.test == 10:
        # stress test, nothing reliable to check
        ret = 0
    elif args.test == 11:
        # stress test, nothing reliable to check
        ret = 0
    else:
        print("Invalid test case")
        sys.exit(1)

    if t is not None:
        ret = t.parse()

    for h in handle.values():
        traces.remove_trace(h)

    sys.exit(ret)



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only

import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *


def check_ust_test_demo2_event(event_line, expected_int_field_value):
    match = re.search(r".*ust_tests_demo2:loop.*", event_line)
    if match is None:
        return False
    match = re.search(r".*intfield = (\d+)", event_line)
    if match is None or int(match.group(1)) != expected_int_field_value:
        return False
    match = re.search(r".*longfield = (\d+)", event_line)
    if match is None or int(match.group(1)) != expected_int_field_value:
        return False
    match = re.search(r".*netintfield = (\d+)", event_line)
    if match is None or int(match.group(1)) != expected_int_field_value:
        return False
    match = re.search(r".*intfield2 = 0x(\d+)", event_line)
    if match is None or int(match.group(1)) != expected_int_field_value:
        return False
    match = re.search(r".*netintfieldhex = 0x(\d+)", event_line)
    if match is None or int(match.group(1)) != expected_int_field_value:
        return False
    match = re.search(r".*floatfield = (\d+)", event_line)
    if match is None or int(match.group(1)) != 2222:
        return False
    match = re.search(r".*doublefield = (\d+)", event_line)
    if match is None or int(match.group(1)) != 2:
        return False
    match = re.search(r".*_seqfield1_length = (\d+)", event_line)
    if match is None or int(match.group(1)) != 4:
        return False
    match = re.search(r".*seqfield1 = \[ \[0\] = (\d+), \[1\] = (\d+), \[2\] = (\d+), \[3\] = (\d+) \]", event_line)
    if match is None or int(match.group(1)) != 116 or int(match.group(2)) != 101 or int(match.group(3)) != 115 or int(match.group(4)) != 116:
        return False
    match = re.search(r".*arrfield1 = \[ \[0\] = (\d), \[1\] = (\d), \[2\] = (\d) \]", event_line)
    if match is None or int(match.group(1)) != 1 or int(match.group(2)) != 2 or int(match.group(3)) != 3:
        return False
    match = re.search(r".*arrfield2 = \"([a-z]*)\"", event_line)
    if match is None or match.group(1) != "test":
        return False
    match = re.search(r".*_seqfield2_length = (\d+)", event_line)
    if match is None or int(match.group(1)) != 4:
        return False
    match = re.search(r".*seqfield2 = \"([a-z]*)\"", event_line)
    if match is None or match.group(1) != "test":
        return False
    match = re.search(r".*stringfield = \"([a-z]*)\"", event_line)
    if match is None or match.group(1) != "test":
        return False

    return True

NR_TESTS = 0
DYNAMIC_TEST_ENABLED = False

test_executables = [test_path + "demo_static", test_path + "demo_builtin"]
if os.path.exists(test_path + "demo"):
    test_executables.append(test_path + "demo_preload")
    NR_TESTS = 2
    DYNAMIC_TEST_ENABLED = True

# Only enable tests that were compiled successfully
test_executables = [executable for executable in test_executables if os.path.exists(executable)]

NR_TESTS += len(test_executables) * 10

current_test = 1
print("1..{0}".format(NR_TESTS))

if NR_TESTS == 0:
    print("# No test binary found")
    exit(-1)

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("No sessiond running. Please make sure you are running this test with the \"run\" shell script and verify that the lttng tools are properly installed.")

if DYNAMIC_TEST_ENABLED:
    session_info = create_session()
    enable_ust_tracepoint_event(session_info, "ust_tests_demo*")
    start_session(session_info)

    # Dry run, no events should be logged
    demo_process = subprocess.Popen(test_path + "demo", stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    demo_process.wait()
    stop_session(session_info)

    print_test_result(demo_process.returncode == 0, current_test,\
                          "Running application dynamically linked to providers, no preload")
    current_test += 1
    trace_path = os.path.join(session_info.trace_path, "ust", "uid")
    print_test_result(not os.path.exists(trace_path), current_test,\
                          "No events logged when running demo application without preloading providers")
    current_test += 1

    shutil.rmtree(session_info.tmp_directory)

for executable in test_executables:
    executable_name = os.path.basename(executable)
    session_info = create_session()
    enable_ust_tracepoint_event(session_info, "ust_tests_demo*")
    start_session(session_info)

    demo_process = subprocess.Popen(executable, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    demo_process.wait()
    stop_session(session_info)

    trace_found = os.path.exists(session_info.trace_path)
    print_test_result(trace_found, current_test,\
                          "{0}, resulting trace found".format(executable_name))
    current_test += 1

    if not trace_found:
        print("# Skipping " + executable_name + " trace verification tests")
        continue

    try:
        babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except FileNotFoundError:
        bail("Could not open babeltrace. Please make sure it is installed.")

    # We should find 8 events in the resulting trace
    event_entries = []
    for event_line in babeltrace_process.stdout:
        event_line = event_line.decode('utf-8').replace("\n", "")
        event_entries.append(event_line)

    if len(event_entries) != 8:
        bail("{0}, wrong number of events found in resulting trace.".format(executable_name))

    shutil.rmtree(session_info.tmp_directory)

    print_test_result(len(event_entries) == 8, current_test,\
                          "{0}, total number of events logged is correct".format(executable_name))
    current_test += 1

    # Check each loop event
    match = re.search(r".*ust_tests_demo:starting.*value = (\d+) ", event_entries[0])
    print_test_result(match is not None and (int(match.group(1)) == 123), current_test,\
                          "{0}, ust_tests_demo:starting event found in trace with a correct integer argument".format(executable_name))
    current_test += 1

    for i in range(5):
        print_test_result(check_ust_test_demo2_event(event_entries[i+1], i), current_test,\
                              "{0}, ust_tests_demo2:loop event found in trace and arguments are correct, iteration ".format(executable_name)\
                              + str(i + 1))
        current_test += 1

    match = re.search(r".*ust_tests_demo:done.*value = (\d+)", event_entries[6])
    print_test_result(match is not None and (int(match.group(1)) == 456), current_test,\
                          "{0}, ust_tests_demo:done event found in resulting trace with a correct integer argument".format(executable_name))
    current_test += 1

    match = re.search(r".*ust_tests_demo3:done.*value = (\d+)", event_entries[7])
    print_test_result(match is not None and (int(match.group(1)) == 42), current_test,\
                          "{0}, ust_tests_demo3:done event found in resulting trace with a correct integer argument".format(executable_name))
    current_test += 1



#!/usr/bin/env python3
#
# Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
#
# SPDX-License-Identifier: GPL-2.0-only

import os
import subprocess
import re
import shutil
import sys

test_path = os.path.dirname(os.path.abspath(__file__)) + "/"
test_utils_path = test_path
for i in range(4):
    test_utils_path = os.path.dirname(test_utils_path)
test_utils_path = test_utils_path + "/utils"
sys.path.append(test_utils_path)
from test_utils import *


normal_exit_message = "exit-fast tracepoint normal exit"
suicide_exit_message = "exit-fast tracepoint suicide"
NR_TESTS = 5
current_test = 1
print("1..{0}".format(NR_TESTS))

# Check if a sessiond is running... bail out if none found.
if session_daemon_alive() == 0:
    bail("No sessiond running. Please make sure you are running this test with the \"run\" shell script and verify that the lttng tools are properly installed.")

session_info = create_session()
enable_ust_tracepoint_event(session_info, "ust_tests_exitfast*")
start_session(session_info)

test_env = os.environ.copy()
test_env["LTTNG_UST_REGISTER_TIMEOUT"] = "-1"

exit_fast_process = subprocess.Popen(test_path + "exit-fast", stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, env=test_env)
exit_fast_process.wait()

print_test_result(exit_fast_process.returncode == 0, current_test, "Test application exited normally")
current_test += 1

exit_fast_process = subprocess.Popen([test_path + "exit-fast", "suicide"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, env=test_env)
exit_fast_process.wait()

stop_session(session_info)

# Check both events (normal exit and suicide messages) are present in the resulting trace
try:
    babeltrace_process = subprocess.Popen(["babeltrace", session_info.trace_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
except FileNotFoundError:
    bail("Could not open babeltrace. Please make sure it is installed.")

event_lines = []
for event_line in babeltrace_process.stdout:
    event_line = event_line.decode('utf-8').replace("\n", "")
    event_lines.append(event_line)
babeltrace_process.wait()

print_test_result(babeltrace_process.returncode == 0, current_test, "Resulting trace is readable")
current_test += 1

if babeltrace_process.returncode != 0:
    bail("Unreadable trace; can't proceed with analysis.")

print_test_result(len(event_lines) == 2, current_test, "Correct number of events found in resulting trace")
current_test += 1

if len(event_lines) != 2:
    bail("Unexpected number of events found in resulting trace (" + session_info.trace_path + ")." )

match = re.search(r".*message = \"(.*)\"", event_lines[0])
print_test_result(match is not None and match.group(1) == normal_exit_message, current_test,\
                      "Tracepoint message generated during normal exit run is present in trace and has the expected value")
current_test += 1

match = re.search(r".*message = \"(.*)\"", event_lines[1])
print_test_result(match is not None and match.group(1) == suicide_exit_message, current_test,\
                      "Tracepoint message generated during suicide run is present in trace and has the expected value")
current_test += 1

shutil.rmtree(session_info.tmp_directory)
